module ExactSampler

import Anneal
using MathOptInterface
const MOI = MathOptInterface

Anneal.@anew Optimizer begin
    name = "Exact Sampler"
    sense = :min
    domain = :bool
end

function Anneal.sample(sampler::Optimizer{T}) where {T}
    # ~*~ Retrieve Attributes ~*~ #
    n = MOI.get(sampler, MOI.NumberOfVariables())
    N = 2^n - 1

    # ~*~ Sample Random States ~*~ #
    results = @timed Vector{Int}[digits(i; base=2, pad=n) for i = 0:N]
    samples = results.value

    # ~*~ Write Solution Metadata ~*~ #
    metadata = Dict{String,Any}(
        "time" => Dict{String,Any}(
            "sample" => results.time,
        )
    )

    # ~*~ Return Sample Set ~*~ #
    return Anneal.SampleSet{Int,T}(sampler, samples, metadata)
end

end # module